@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2004 - 2010 Red Hat, Inc.
+ * Copyright (C) 2004 - 2011 Red Hat, Inc.
  * Copyright (C) 2005 - 2008 Novell, Inc.
  */
 
@@ -32,7 +32,6 @@
 #include <gdk/gdkx.h>
 
 #include <gconf/gconf-client.h>
-#include <glade/glade.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <net/ethernet.h>
@@ -45,10 +44,8 @@
 #include <nm-device.h>
 #include <NetworkManager.h>
 #include <nm-active-connection.h>
-#include <nm-remote-settings-system.h>
-
-#include "applet-dbus-manager.h"
-#include "nma-gconf-settings.h"
+#include <nm-remote-settings.h>
+#include "applet-agent.h"
 
 #define NM_TYPE_APPLET			(nma_get_type())
 #define NM_APPLET(object)		(G_TYPE_CHECK_INSTANCE_CAST((object), NM_TYPE_APPLET, NMApplet))
@@ -65,6 +62,7 @@ typedef struct
 #define APPLET_PREFS_PATH "/apps/nm-applet"
 #define PREF_DISABLE_CONNECTED_NOTIFICATIONS      APPLET_PREFS_PATH "/disable-connected-notifications"
 #define PREF_DISABLE_DISCONNECTED_NOTIFICATIONS   APPLET_PREFS_PATH "/disable-disconnected-notifications"
+#define PREF_DISABLE_VPN_NOTIFICATIONS            APPLET_PREFS_PATH "/disable-vpn-notifications"
 #define PREF_DISABLE_WIFI_CREATE                  APPLET_PREFS_PATH "/disable-wifi-create"
 #define PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE APPLET_PREFS_PATH "/suppress-wireless-networks-available"
 
@@ -83,15 +81,15 @@ typedef struct
 	GObject parent_instance;
 
 	GMainLoop *loop;
-	NMClient *nm_client;
+	DBusGConnection *bus;
+	DBusGConnection *session_bus;
 
-	NMRemoteSettingsSystem *system_settings;
-	NMAGConfSettings *gconf_settings;
+	NMClient *nm_client;
+	NMRemoteSettings *settings;
+	AppletAgent *agent;
 
 	GConfClient *	gconf_client;
-	char	*		glade_file;
-
-	guint update_timestamps_id;
+	char	*		ui_file;
 
 	/* Permissions */
 	NMClientPermissionResult permissions[NM_CLIENT_PERMISSION_LAST + 1];
@@ -102,6 +100,7 @@ typedef struct
 	NMADeviceClass *gsm_class;
 	NMADeviceClass *cdma_class;
 	NMADeviceClass *bt_class;
+	NMADeviceClass *wimax_class;
 
 	/* Data model elements */
 	guint			update_icon_id;
@@ -157,6 +156,8 @@ typedef struct
 	guint           wifi_enabled_toggled_id;
 	GtkWidget *		wwan_enabled_item;
 	guint           wwan_enabled_toggled_id;
+	GtkWidget *		wimax_enabled_item;
+	guint           wimax_enabled_toggled_id;
 
 	GtkWidget *     notifications_enabled_item;
 	guint           notifications_enabled_toggled_id;
@@ -164,9 +165,12 @@ typedef struct
 	GtkWidget *		info_menu_item;
 	GtkWidget *		connections_menu_item;
 
-	GladeXML *		info_dialog_xml;
+	GtkBuilder *	info_dialog_ui;
 	NotifyNotification*	notification;
 	gboolean        notify_actions;
+
+	/* Tracker objects for secrets requests */
+	GSList *        secrets_reqs;
 } NMApplet;
 
 typedef void (*AppletNewAutoConnectionCallback) (NMConnection *connection,
@@ -174,6 +178,35 @@ typedef void (*AppletNewAutoConnectionCallback) (NMConnection *connection,
                                                  gboolean canceled,
                                                  gpointer user_data);
 
+typedef struct _SecretsRequest SecretsRequest;
+typedef void (*SecretsRequestFreeFunc) (SecretsRequest *req);
+
+struct _SecretsRequest {
+	size_t totsize;
+	gpointer reqid;
+	char *setting_name;
+	char **hints;
+	guint32 flags;
+	NMApplet *applet;
+	AppletAgentSecretsCallback callback;
+	gpointer callback_data;
+
+	NMConnection *connection;
+
+	/* Class-specific stuff */
+	SecretsRequestFreeFunc free_func;
+};
+
+void applet_secrets_request_set_free_func (SecretsRequest *req,
+                                           SecretsRequestFreeFunc free_func);
+void applet_secrets_request_complete (SecretsRequest *req,
+                                      GHashTable *settings,
+                                      GError *error);
+void applet_secrets_request_complete_setting (SecretsRequest *req,
+                                              const char *setting_name,
+                                              GError *error);
+void applet_secrets_request_free (SecretsRequest *req);
+
 struct NMADeviceClass {
 	gboolean       (*new_auto_connection)  (NMDevice *device,
 	                                        gpointer user_data,
@@ -205,14 +238,8 @@ struct NMADeviceClass {
 	                                        NMApplet *applet,
 	                                        gpointer user_data);
 
-	gboolean       (*get_secrets)          (NMDevice *device,
-	                                        NMSettingsConnectionInterface *connection,
-	                                        NMActiveConnection *active_connection,
-	                                        const char *setting_name,
-	                                        const char **hints,
-	                                        NMANewSecretsRequestedFunc callback,
-	                                        gpointer callback_data,
-	                                        NMApplet *applet,
+	size_t         secrets_request_size;
+	gboolean       (*get_secrets)          (SecretsRequest *req,
 	                                        GError **error);
 };
 
@@ -222,7 +249,7 @@ NMApplet *nm_applet_new (GMainLoop *loop);
 
 void applet_schedule_update_icon (NMApplet *applet);
 
-NMSettingsInterface *applet_get_settings (NMApplet *applet);
+NMRemoteSettings *applet_get_settings (NMApplet *applet);
 
 GSList *applet_get_all_connections (NMApplet *applet);
 
@@ -251,7 +278,9 @@ applet_menu_item_create_device_item_helper (NMDevice *device,
                                             NMApplet *applet,
                                             const gchar *text);
 
-NMSettingsConnectionInterface *applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet);
+NMRemoteConnection *applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet);
+
+NMDevice *applet_get_device_for_connection (NMApplet *applet, NMConnection *connection);
 
 void applet_do_notify (NMApplet *applet,
                        NotifyUrgency urgency,
@@ -281,4 +310,8 @@ GdkPixbuf * nma_icon_check_and_load (const char *name,
                                      GdkPixbuf **icon,
                                      NMApplet *applet);
 
+gboolean applet_wifi_connect_to_hidden_network (NMApplet *applet);
+gboolean applet_wifi_create_wifi_network (NMApplet *applet);
+gboolean applet_wifi_can_create_wifi_network (NMApplet *applet);
+
 #endif
