@@ -27,7 +27,6 @@
 #include <unistd.h>
 
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 
@@ -44,7 +43,7 @@
 #include <nm-setting-serial.h>
 #include <nm-vpn-plugin-ui-interface.h>
 #include <nm-utils.h>
-#include <nm-settings-system-interface.h>
+#include <nm-remote-settings.h>
 
 #include "ce-page.h"
 #include "page-wired.h"
@@ -54,8 +53,6 @@
 #include "page-vpn.h"
 #include "nm-connection-editor.h"
 #include "nm-connection-list.h"
-#include "gconf-helpers.h"
-#include "utils.h"
 #include "vpn-helpers.h"
 #include "ce-polkit-button.h"
 
@@ -107,14 +104,14 @@ error_dialog (GtkWindow *parent, const char *heading, const char *format, ...)
 	gtk_widget_destroy (dialog);
 }
 
-static NMSettingsConnectionInterface *
+static NMRemoteConnection *
 get_active_connection (GtkTreeView *treeview)
 {
 	GtkTreeSelection *selection;
 	GList *selected_rows;
 	GtkTreeModel *model = NULL;
 	GtkTreeIter iter;
-	NMSettingsConnectionInterface *connection = NULL;
+	NMRemoteConnection *connection = NULL;
 
 	selection = gtk_tree_view_get_selection (treeview);
 	selected_rows = gtk_tree_selection_get_selected_rows (selection, &model);
@@ -151,7 +148,7 @@ get_treeview_for_type (NMConnectionList *list, GType ctype)
 }
 
 static GtkListStore *
-get_model_for_connection (NMConnectionList *list, NMSettingsConnectionInterface *connection)
+get_model_for_connection (NMConnectionList *list, NMRemoteConnection *connection)
 {
 	NMSettingConnection *s_con;
 	GtkTreeView *treeview;
@@ -171,10 +168,8 @@ get_model_for_connection (NMConnectionList *list, NMSettingsConnectionInterface
 		str_type = NM_SETTING_GSM_SETTING_NAME;
 
 	treeview = get_treeview_for_type (list, nm_connection_lookup_setting_type (str_type));
-	if (!treeview) {
-		g_warning ("No registered treeview for connection type '%s'", str_type);
+	if (!treeview)
 		return NULL;
-	}
 
 	model = gtk_tree_view_get_model (treeview);
 	if (GTK_IS_TREE_MODEL_SORT (model))
@@ -185,7 +180,7 @@ get_model_for_connection (NMConnectionList *list, NMSettingsConnectionInterface
 
 static gboolean
 get_iter_for_connection (GtkTreeModel *model,
-                         NMSettingsConnectionInterface *connection,
+                         NMRemoteConnection *connection,
                          GtkTreeIter *iter)
 {
 	GtkTreeIter temp_iter;
@@ -195,7 +190,7 @@ get_iter_for_connection (GtkTreeModel *model,
 		return FALSE;
 
 	do {
-		NMSettingsConnectionInterface *candidate = NULL;
+		NMRemoteConnection *candidate = NULL;
 
 		gtk_tree_model_get (model, &temp_iter, COL_CONNECTION, &candidate, -1);
 		if (candidate && (candidate == connection)) {
@@ -283,7 +278,7 @@ out:
 static void
 update_connection_row (GtkListStore *store,
                        GtkTreeIter *iter,
-                       NMSettingsConnectionInterface *connection)
+                       NMRemoteConnection *connection)
 {
 	NMSettingConnection *s_con;
 	char *last_used;
@@ -311,63 +306,29 @@ typedef void (*DeleteResultFunc) (NMConnectionList *list,
 
 typedef struct {
 	NMConnectionList *list;
-	NMSettingsConnectionInterface *original;
+	NMRemoteConnection *original;
 	NMConnectionEditor *editor;
 	DeleteResultFunc callback;
 	gpointer callback_data;
 } DeleteInfo;
 
 static void
-delete_cb (NMSettingsConnectionInterface *connection_iface,
+delete_cb (NMRemoteConnection *remote,
            GError *error,
            gpointer user_data)
 {
 	DeleteInfo *info = user_data;
-	NMConnection *connection = NM_CONNECTION (connection_iface);
-	NMConnectionScope scope;
 
 	if (info->editor)
 		nm_connection_editor_set_busy (info->editor, FALSE);
 
-	scope = nm_connection_get_scope (connection);
-	if (!error && (scope == NM_CONNECTION_SCOPE_USER)) {
-		NMSettingConnection *s_con;
-		NMSettingVPN *s_vpn;
-		NMVpnPluginUiInterface *plugin;
-		GError *vpn_error = NULL;
-
-		s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
-		g_assert (s_con);
-
-		/* Clean up VPN secrets and any plugin-specific data */
-		if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_VPN_SETTING_NAME))
-			goto done;
-
-		s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
-		if (!s_vpn)
-			goto done;
-
-		plugin = vpn_get_plugin_by_service (nm_setting_vpn_get_service_type (s_vpn));
-		if (!plugin)
-			goto done;
-
-		if (!nm_vpn_plugin_ui_interface_delete_connection (plugin, connection, &vpn_error)) {
-			g_warning ("%s: couldn't clean up VPN connection on delete: (%d) %s",
-			           __func__,
-			           vpn_error ? vpn_error->code : -1,
-			           (vpn_error && vpn_error->message) ? vpn_error->message : "unknown");
-			g_clear_error (&vpn_error);
-		}
-	}
-
-done:
 	info->callback (info->list, error, info->callback_data);
 	g_free (info);
 }
 
 static void
 delete_connection (NMConnectionList *list,
-                   NMSettingsConnectionInterface *connection,
+                   NMRemoteConnection *connection,
                    DeleteResultFunc callback,
                    gpointer user_data)
 {
@@ -385,7 +346,7 @@ delete_connection (NMConnectionList *list,
 	if (editor)
 		nm_connection_editor_set_busy (editor, TRUE);
 
-	nm_settings_connection_interface_delete (connection, delete_cb, info);
+	nm_remote_connection_delete (connection, delete_cb, info);
 }
 
 /**********************************************/
@@ -403,22 +364,14 @@ typedef struct {
 } AddInfo;
 
 static void
-add_cb (NMSettingsInterface *settings,
+add_cb (NMRemoteSettings *settings,
+        NMRemoteConnection *connection,
         GError *error,
         gpointer user_data)
 {
 	AddInfo *info = user_data;
-	NMConnection *connection;
 
 	nm_connection_editor_set_busy (info->editor, FALSE);
-
-	if (!error) {
-		/* Let the VPN plugin save its secrets */
-		connection = nm_connection_editor_get_connection (info->editor);
-		if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_USER)
-			nm_connection_editor_save_vpn_secrets (info->editor);
-	}
-
 	info->callback (info->list, error, info->callback_data);
 	g_free (info);
 }
@@ -429,9 +382,6 @@ add_connection (NMConnectionList *self,
                 AddResultFunc callback,
                 gpointer callback_data)
 {
-	NMSettingsInterface *settings = NULL;
-	NMConnection *connection;
-	NMConnectionScope scope;
 	AddInfo *info;
 
 	info = g_malloc0 (sizeof (AddInfo));
@@ -440,33 +390,26 @@ add_connection (NMConnectionList *self,
 	info->callback = callback;
 	info->callback_data = callback_data;
 
-	connection = nm_connection_editor_get_connection (editor);
-	g_assert (connection);
-	scope = nm_connection_get_scope (connection);
-	if (scope == NM_CONNECTION_SCOPE_SYSTEM)
-		settings = NM_SETTINGS_INTERFACE (self->system_settings);
-	else if (scope == NM_CONNECTION_SCOPE_USER)
-		settings = NM_SETTINGS_INTERFACE (self->gconf_settings);
-	else
-		g_assert_not_reached ();
-
 	nm_connection_editor_set_busy (editor, TRUE);
-	nm_settings_interface_add_connection (settings, connection, add_cb, info);
+
+	nm_remote_settings_add_connection (self->settings,
+	                                   nm_connection_editor_get_connection (editor),
+	                                   add_cb,
+	                                   info);
 }
 
 /**********************************************/
 /* Connection updating */
 
 typedef void (*UpdateResultFunc) (NMConnectionList *list,
-                                  NMSettingsConnectionInterface *connection,
+                                  NMRemoteConnection *connection,
                                   GError *error,
                                   gpointer user_data);
 
 typedef struct {
 	NMConnectionList *list;
 	NMConnectionEditor *editor;
-	NMSettingsConnectionInterface *connection;
-	NMConnectionScope orig_scope;
+	NMRemoteConnection *connection;
 	UpdateResultFunc callback;
 	gpointer callback_data;
 } UpdateInfo;
@@ -480,46 +423,12 @@ update_complete (UpdateInfo *info, GError *error)
 }
 
 static void
-update_remove_result_cb (NMConnectionList *list,
-                         GError *error,
-                         gpointer user_data)
-{
-	UpdateInfo *info = user_data;
-
-	update_complete (info, error);
-}
-
-static void
-update_add_result_cb (NMConnectionList *list, GError *error, gpointer user_data)
-{
-	UpdateInfo *info = user_data;
-
-	if (error) {
-		/* set the old scope back on the connection */
-		nm_connection_set_scope (NM_CONNECTION (info->connection), info->orig_scope);
-		update_complete (info, error);
-		return;
-	}
-
-	/* Now try to remove the original connection */
-	delete_connection (list, info->connection, update_remove_result_cb, info);
-}
-
-static void
-update_cb (NMSettingsConnectionInterface *connection,
-           GError *error,
-           gpointer user_data)
+update_cb (NMRemoteConnection *connection, GError *error, gpointer user_data)
 {
 	UpdateInfo *info = user_data;
 
 	nm_connection_editor_set_busy (info->editor, FALSE);
 
-	if (!error) {
-		/* Save user-connection vpn secrets */
-		if (nm_connection_get_scope (NM_CONNECTION (connection)) == NM_CONNECTION_SCOPE_USER)
-			nm_connection_editor_save_vpn_secrets (info->editor);
-	}
-
 	/* Clear secrets so they don't lay around in memory; they'll get requested
 	 * again anyway next time the connection is edited.
 	 */
@@ -531,55 +440,34 @@ update_cb (NMSettingsConnectionInterface *connection,
 static void
 update_connection (NMConnectionList *list,
                    NMConnectionEditor *editor,
-                   NMSettingsConnectionInterface *connection,
-                   NMConnectionScope orig_scope,
+                   NMRemoteConnection *connection,
                    UpdateResultFunc callback,
                    gpointer user_data)
 {
-	NMConnectionScope new_scope;
 	UpdateInfo *info;
+	GHashTable *new_settings;
+	GError *error = NULL;
 
 	info = g_malloc0 (sizeof (UpdateInfo));
 	info->list = list;
 	info->editor = editor;
 	info->connection = g_object_ref (connection);
-	info->orig_scope = orig_scope;
 	info->callback = callback;
 	info->callback_data = user_data;
 
-	new_scope = nm_connection_get_scope (NM_CONNECTION (connection));
-	if (new_scope == orig_scope) {
-		/* The easy part: Connection is just updated and has the same scope */
-		GHashTable *new_settings;
-		GError *error = NULL;
-
-		/* System connections need the certificates filled because the
-		 * applet private values that we use to store the path to certificates
-		 * and private keys don't go through D-Bus; they are private of course!
-		 */
-		if (new_scope == NM_CONNECTION_SCOPE_SYSTEM) {
-			new_settings = nm_connection_to_hash (NM_CONNECTION (connection));
-			if (!nm_connection_replace_settings (NM_CONNECTION (connection),
-			                                     new_settings,
-			                                     &error)) {
-				update_complete (info, error);
-				g_error_free (error);
-				return;
-			}
-		}
-
-		/* Update() actually saves the connection settings to backing storage,
-		 * either GConf or over D-Bus.
-		 */
-		nm_connection_editor_set_busy (editor, TRUE);
-		nm_settings_connection_interface_update (connection, update_cb, info);
-	} else {
-		/* The hard part: Connection scope changed:
-		 * Add the modified connection to the new settings service, then delete
-		 * the original connection from the old settings service.
-		 */
-		add_connection (list, editor, update_add_result_cb, info);
+	/* Connections need the certificates filled because the
+	 * applet private values that we use to store the path to certificates
+	 * and private keys don't go through D-Bus; they are private of course!
+	 */
+	new_settings = nm_connection_to_hash (NM_CONNECTION (connection), NM_SETTING_HASH_FLAG_ALL);
+	if (!nm_connection_replace_settings (NM_CONNECTION (connection), new_settings, &error)) {
+		update_complete (info, error);
+		g_error_free (error);
+		return;
 	}
+
+	nm_connection_editor_set_busy (editor, TRUE);
+	nm_remote_connection_commit_changes (connection, update_cb, info);
 }
 
 /**********************************************/
@@ -622,7 +510,8 @@ add_response_cb (NMConnectionEditor *editor, gint response, GError *error, gpoin
 			return;
 		} else {
 			error_dialog (GTK_WINDOW (editor->window),
-			              _("Error editing connection: property '%s' / '%s' invalid: %d"),
+			              _("Error saving connection"),
+			              _("The property '%s' / '%s' is invalid: %d"),
 			              g_type_name (nm_connection_lookup_setting_type_by_quark (add_error->domain)),
 			              (add_error && add_error->message) ? add_error->message : "(unknown)",
 			              add_error ? add_error->code : -1);
@@ -665,7 +554,7 @@ really_add_connection (NMConnection *connection,
 		return;
 	}
 
-	editor = nm_connection_editor_new (connection, info->list->system_settings, &error);
+	editor = nm_connection_editor_new (connection, info->list->nm_client, &error);
 	if (!editor) {
 		g_object_unref (connection);
 
@@ -688,8 +577,7 @@ page_get_connections (gpointer user_data)
 {
 	ActionInfo *info = (ActionInfo *) user_data;
 
-	return g_slist_concat (nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (info->list->system_settings)),
-	                       nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (info->list->gconf_settings)));
+	return nm_remote_settings_list_connections (info->list->settings);
 }
 
 static void
@@ -715,12 +603,11 @@ add_clicked (GtkButton *button, gpointer user_data)
 typedef struct {
 	NMConnectionList *list;
 	NMConnectionEditor *editor;
-	NMConnectionScope orig_scope;
 } EditInfo;
 
 static void
 connection_updated_cb (NMConnectionList *list,
-                       NMSettingsConnectionInterface *connection,
+                       NMRemoteConnection *connection,
                        GError *error,
                        gpointer user_data)
 {
@@ -764,8 +651,7 @@ edit_done_cb (NMConnectionEditor *editor, gint response, GError *error, gpointer
 			/* Save the connection to backing storage */
 			update_connection (info->list,
 			                   editor,
-			                   NM_SETTINGS_CONNECTION_INTERFACE (connection),
-			                   info->orig_scope,
+			                   NM_REMOTE_CONNECTION (connection),
 			                   connection_updated_cb,
 			                   info);
 		} else {
@@ -775,7 +661,7 @@ edit_done_cb (NMConnectionEditor *editor, gint response, GError *error, gpointer
 			           g_type_name (nm_connection_lookup_setting_type_by_quark (edit_error->domain)),
 			           edit_error->message, edit_error->code);
 			connection_updated_cb (info->list,
-			                       NM_SETTINGS_CONNECTION_INTERFACE (connection),
+			                       NM_REMOTE_CONNECTION (connection),
 			                       edit_error,
 			                       info);
 			g_error_free (edit_error);
@@ -798,7 +684,7 @@ edit_done_cb (NMConnectionEditor *editor, gint response, GError *error, gpointer
 static void
 do_edit (ActionInfo *info)
 {
-	NMSettingsConnectionInterface *connection;
+	NMRemoteConnection *connection;
 	NMConnectionEditor *editor;
 	EditInfo *edit_info;
 	GError *error = NULL;
@@ -814,7 +700,7 @@ do_edit (ActionInfo *info)
 		return;
 	}
 
-	editor = nm_connection_editor_new (NM_CONNECTION (connection), info->list->system_settings, &error);
+	editor = nm_connection_editor_new (NM_CONNECTION (connection), info->list->nm_client, &error);
 	if (!editor) {
 		error_dialog (info->list_window,
 		              _("Could not edit connection"),
@@ -826,7 +712,6 @@ do_edit (ActionInfo *info)
 	edit_info = g_malloc0 (sizeof (EditInfo));
 	edit_info->list = info->list;
 	edit_info->editor = editor;
-	edit_info->orig_scope = nm_connection_get_scope (NM_CONNECTION (connection));
 
 	g_signal_connect (editor, "done", G_CALLBACK (edit_done_cb), edit_info);
 	g_hash_table_insert (info->list->editors, connection, editor);
@@ -849,7 +734,7 @@ static void
 delete_clicked (GtkButton *button, gpointer user_data)
 {
 	ActionInfo *info = (ActionInfo *) user_data;
-	NMSettingsConnectionInterface *connection;
+	NMRemoteConnection *connection;
 	NMConnectionEditor *editor;
 	NMSettingConnection *s_con;
 	GtkWidget *dialog;
@@ -885,8 +770,12 @@ delete_clicked (GtkButton *button, gpointer user_data)
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	if (result == GTK_RESPONSE_YES)
-		delete_connection (info->list, connection, delete_result_cb, GTK_WINDOW (info->list->dialog));
+	if (result == GTK_RESPONSE_YES) {
+		delete_connection (info->list,
+		                   connection,
+		                   delete_result_cb,
+		                   GTK_WINDOW (info->list->dialog));
+	}
 }
 
 static void
@@ -895,30 +784,21 @@ pk_button_selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
 	ActionInfo *info = (ActionInfo *) user_data;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	NMSettingsConnectionInterface *connection;
+	NMRemoteConnection *connection;
 	NMSettingConnection *s_con;
-	NMConnectionScope scope;
 	gboolean sensitive = FALSE;
-	gboolean use_polkit = FALSE;
 
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-		goto done;
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		connection = get_active_connection (info->treeview);
+		if (connection) {
+			s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (connection),
+			                                                           NM_TYPE_SETTING_CONNECTION);
+			g_assert (s_con);
+	
+			sensitive = !nm_setting_connection_get_read_only (s_con);
+		}
+	}
 
-	connection = get_active_connection (info->treeview);
-	if (!connection)
-		goto done;
-
-	scope = nm_connection_get_scope (NM_CONNECTION (connection));
-	use_polkit = (scope == NM_CONNECTION_SCOPE_SYSTEM);
-
-	s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (connection),
-	                                                           NM_TYPE_SETTING_CONNECTION);
-	g_assert (s_con);
-
-	sensitive = !nm_setting_connection_get_read_only (s_con);
-
-done:
-	ce_polkit_button_set_use_polkit (CE_POLKIT_BUTTON (info->button), use_polkit);
 	ce_polkit_button_set_master_sensitive (CE_POLKIT_BUTTON (info->button), sensitive);
 }
 
@@ -927,7 +807,7 @@ vpn_list_selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
 {
 	ActionInfo *info = (ActionInfo *) user_data;
 	NMVpnPluginUiInterface *plugin;
-	NMSettingsConnectionInterface *connection;
+	NMRemoteConnection *connection;
 	NMSettingVPN *s_vpn;
 	const char *service_type;
 	GtkTreeIter iter;
@@ -983,11 +863,8 @@ import_success_cb (NMConnection *connection, gpointer user_data)
 	if (!s) {
 		GSList *connections;
 
-		connections = nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (info->list->system_settings));
-		connections = g_slist_concat (connections,
-		                              nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (info->list->gconf_settings)));
-
-		s = utils_next_available_name (connections, _("VPN connection %d"));
+		connections = nm_remote_settings_list_connections (info->list->settings);
+		s = ce_page_get_next_available_name (connections, _("VPN connection %d"));
 		g_object_set (s_con, NM_SETTING_CONNECTION_ID, s, NULL);
 		g_free (s);
 
@@ -1028,7 +905,7 @@ import_success_cb (NMConnection *connection, gpointer user_data)
 		return;
 	}
 
-	editor = nm_connection_editor_new (connection, info->list->system_settings, &error);
+	editor = nm_connection_editor_new (connection, info->list->nm_client, &error);
 	if (!editor) {
 		g_object_unref (connection);
 		error_dialog (info->list_window,
@@ -1054,7 +931,7 @@ static void
 export_vpn_cb (GtkButton *button, gpointer user_data)
 {
 	ActionInfo *info = (ActionInfo *) user_data;
-	NMSettingsConnectionInterface *connection;
+	NMRemoteConnection *connection;
 
 	connection = get_active_connection (info->treeview);
 	if (connection)
@@ -1108,16 +985,13 @@ dispose (GObject *object)
 
 	if (list->gui)
 		g_object_unref (list->gui);
-	if (list->client)
-		g_object_unref (list->client);
+	if (list->nm_client)
+		g_object_unref (list->nm_client);
 
 	g_slist_free (list->treeviews);
 
-	if (list->gconf_settings)
-		g_object_unref (list->gconf_settings);
-
-	if (list->system_settings)
-		g_object_unref (list->system_settings);
+	if (list->settings)
+		g_object_unref (list->settings);
 
 	G_OBJECT_CLASS (nm_connection_list_parent_class)->dispose (object);
 }
@@ -1153,7 +1027,7 @@ add_connection_treeview (NMConnectionList *self, const char *prefix)
 	GtkTreeView *treeview;
 
 	name = g_strdup_printf ("%s_list", prefix);
-	treeview = GTK_TREE_VIEW (glade_xml_get_widget (self->gui, name));
+	treeview = GTK_TREE_VIEW (GTK_WIDGET (gtk_builder_get_object (self->gui, name)));
 	g_free (name);
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 
@@ -1261,33 +1135,38 @@ add_connection_buttons (NMConnectionList *self,
 
 	/* Add */
 	name = g_strdup_printf ("%s_add", prefix);
-	button = glade_xml_get_widget (self->gui, name);
+	button = GTK_WIDGET (gtk_builder_get_object (self->gui, name));
 	g_free (name);
 	info = action_info_new (self, treeview, GTK_WINDOW (self->dialog), NULL);
 	g_signal_connect (button, "clicked", G_CALLBACK (add_clicked), info);
 	if (ctype == NM_TYPE_SETTING_VPN) {
 		GHashTable *plugins;
+		gboolean have_plugins;
 
 		/* disable the "Add..." button if there aren't any VPN plugins */
 		plugins = vpn_get_plugins (NULL);
-		gtk_widget_set_sensitive (button, (plugins && g_hash_table_size (plugins)));
+		have_plugins = plugins && g_hash_table_size (plugins);
+		gtk_widget_set_sensitive (button, have_plugins);
+		if (!have_plugins)
+			gtk_widget_set_tooltip_text (button, _("No VPN plugin available. Please install one to enable this button."));
 	}
 	if (new_func)
 		action_info_set_new_func (info, new_func);
 
 	name = g_strdup_printf ("%s_button_box", prefix);
-	hbox = glade_xml_get_widget (self->gui, name);
+	hbox = GTK_WIDGET (gtk_builder_get_object (self->gui, name));
 	g_free (name);
 
 	/* Edit */
 	info = action_info_new (self, treeview, GTK_WINDOW (self->dialog), NULL);
-	button = ce_polkit_button_new (_("Edit"),
+	button = ce_polkit_button_new (_("_Edit"),
 	                               _("Edit the selected connection"),
-	                               _("Edit..."),
+	                               _("_Edit..."),
 	                               _("Authenticate to edit the selected connection"),
 	                               GTK_STOCK_EDIT,
-	                               self->system_settings,
-	                               NM_SETTINGS_SYSTEM_PERMISSION_CONNECTION_MODIFY);
+	                               self->nm_client,
+	                               NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
 	gtk_box_pack_end (GTK_BOX (hbox), button, TRUE, TRUE, 0);
 
 	action_info_set_button (info, button);
@@ -1298,13 +1177,14 @@ add_connection_buttons (NMConnectionList *self,
 
 	/* Delete */
 	info = action_info_new (self, treeview, GTK_WINDOW (self->dialog), NULL);
-	button = ce_polkit_button_new (_("Delete"),
+	button = ce_polkit_button_new (_("_Delete"),
 	                               _("Delete the selected connection"),
-	                               _("Delete..."),
+	                               _("_Delete..."),
 	                               _("Authenticate to delete the selected connection"),
 	                               GTK_STOCK_DELETE,
-	                               self->system_settings,
-	                               NM_SETTINGS_SYSTEM_PERMISSION_CONNECTION_MODIFY);
+	                               self->nm_client,
+	                               NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
+	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
 	gtk_box_pack_end (GTK_BOX (hbox), button, TRUE, TRUE, 0);
 
 	action_info_set_button (info, button);
@@ -1314,7 +1194,7 @@ add_connection_buttons (NMConnectionList *self,
 
 	/* Import */
 	name = g_strdup_printf ("%s_import", prefix);
-	button = glade_xml_get_widget (self->gui, name);
+	button = GTK_WIDGET (gtk_builder_get_object (self->gui, name));
 	g_free (name);
 	if (button) {
 		gboolean import_supported = FALSE;
@@ -1331,7 +1211,7 @@ add_connection_buttons (NMConnectionList *self,
 
 	/* Export */
 	name = g_strdup_printf ("%s_export", prefix);
-	button = glade_xml_get_widget (self->gui, name);
+	button = GTK_WIDGET (gtk_builder_get_object (self->gui, name));
 	g_free (name);
 	if (button) {
 		info = action_info_new (self, treeview, GTK_WINDOW (self->dialog), button);
@@ -1356,7 +1236,7 @@ add_connection_tab (NMConnectionList *self,
 	int pnum;
 
 	name = g_strdup_printf ("%s_child", prefix);
-	child = glade_xml_get_widget (self->gui, name);
+	child = GTK_WIDGET (gtk_builder_get_object (self->gui, name));
 	g_free (name);
 
 	/* Notebook tab */
@@ -1370,7 +1250,7 @@ add_connection_tab (NMConnectionList *self,
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (label_text), FALSE, FALSE, 0);
 	gtk_widget_show_all (hbox);
 
-	notebook = glade_xml_get_widget (self->gui, "list_notebook");
+	notebook = GTK_WIDGET (gtk_builder_get_object (self->gui, "list_notebook"));
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), child, hbox);
 
 	treeview = add_connection_treeview (self, prefix);
@@ -1411,7 +1291,7 @@ add_connection_tabs (NMConnectionList *self, GType def_type)
 }
 
 static void
-connection_removed (NMSettingsConnectionInterface *connection, gpointer user_data)
+connection_removed (NMRemoteConnection *connection, gpointer user_data)
 {
 	GtkListStore *store = GTK_LIST_STORE (user_data);
 	GtkTreeIter iter;
@@ -1421,9 +1301,7 @@ connection_removed (NMSettingsConnectionInterface *connection, gpointer user_dat
 }
 
 static void
-connection_updated (NMSettingsConnectionInterface *connection,
-                    GHashTable *settings,
-                    gpointer user_data)
+connection_updated (NMRemoteConnection *connection, gpointer user_data)
 {
 	GtkListStore *store = GTK_LIST_STORE (user_data);
 	GtkTreeIter iter;
@@ -1433,8 +1311,8 @@ connection_updated (NMSettingsConnectionInterface *connection,
 }
 
 static void
-connection_added (NMSettingsInterface *settings,
-                  NMSettingsConnectionInterface *connection,
+connection_added (NMRemoteSettings *settings,
+                  NMRemoteConnection *connection,
                   gpointer user_data)
 {
 	NMConnectionList *self = NM_CONNECTION_LIST (user_data);
@@ -1461,8 +1339,8 @@ connection_added (NMSettingsInterface *settings,
 
 	g_free (last_used);
 
-	g_signal_connect (connection, "removed", G_CALLBACK (connection_removed), store);
-	g_signal_connect (connection, "updated", G_CALLBACK (connection_updated), store);
+	g_signal_connect (connection, NM_REMOTE_CONNECTION_REMOVED, G_CALLBACK (connection_removed), store);
+	g_signal_connect (connection, NM_REMOTE_CONNECTION_UPDATED, G_CALLBACK (connection_updated), store);
 }
 
 #define ICON_LOAD(x, y)	\
@@ -1487,9 +1365,11 @@ nm_connection_list_new (GType def_type)
 		return NULL;
 
 	/* load GUI */
-	list->gui = glade_xml_new (GLADEDIR "/nm-connection-editor.glade", "NMConnectionList", NULL);
-	if (!list->gui) {
-		g_warning ("Could not load Glade file for connection list");
+	list->gui = gtk_builder_new ();
+
+	if (!gtk_builder_add_from_file (list->gui, UIDIR "/nm-connection-editor.ui", &error)) {
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
 		goto error;
 	}
 
@@ -1504,8 +1384,8 @@ nm_connection_list_new (GType def_type)
 	ICON_LOAD(list->vpn_icon, "nm-vpn-standalone-lock");
 	ICON_LOAD(list->unknown_icon, "nm-no-connection");
 
-	list->client = gconf_client_get_default ();
-	if (!list->client)
+	list->nm_client = nm_client_new ();
+	if (!list->nm_client)
 		goto error;
 
 	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
@@ -1515,16 +1395,10 @@ nm_connection_list_new (GType def_type)
 		goto error;
 	}
 
-	list->system_settings = nm_remote_settings_system_new (bus);
+	list->settings = nm_remote_settings_new (bus);
 	dbus_g_connection_unref (bus);
-	g_signal_connect (list->system_settings,
-	                  NM_SETTINGS_INTERFACE_NEW_CONNECTION,
-	                  G_CALLBACK (connection_added),
-	                  list);
-
-	list->gconf_settings = nma_gconf_settings_new (NULL);
-	g_signal_connect (list->gconf_settings,
-	                  NM_SETTINGS_INTERFACE_NEW_CONNECTION,
+	g_signal_connect (list->settings,
+	                  NM_REMOTE_SETTINGS_NEW_CONNECTION,
 	                  G_CALLBACK (connection_added),
 	                  list);
 
@@ -1532,7 +1406,7 @@ nm_connection_list_new (GType def_type)
 
 	list->editors = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
 
-	list->dialog = glade_xml_get_widget (list->gui, "NMConnectionList");
+	list->dialog = GTK_WIDGET (gtk_builder_get_object (list->gui, "NMConnectionList"));
 	if (!list->dialog)
 		goto error;
 	g_signal_connect (G_OBJECT (list->dialog), "response", G_CALLBACK (dialog_response_cb), list);
@@ -1568,7 +1442,7 @@ nm_connection_list_set_type (NMConnectionList *self, GType ctype)
 	/* If a notebook page is found that owns the requested type, set it
 	 * as the current page.
 	 */
-	notebook = GTK_NOTEBOOK (glade_xml_get_widget (self->gui, "list_notebook"));
+	notebook = GTK_NOTEBOOK (GTK_WIDGET (gtk_builder_get_object (self->gui, "list_notebook")));
 	for (i = 0; i < gtk_notebook_get_n_pages (notebook); i++) {
 		GtkWidget *child;
 		GType child_type;
