@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
@@ -36,7 +36,6 @@
 #include "page-wired.h"
 #include "page-wired-security.h"
 #include "nm-connection-editor.h"
-#include "gconf-helpers.h"
 
 G_DEFINE_TYPE (CEPageWiredSecurity, ce_page_wired_security, CE_TYPE_PAGE)
 
@@ -72,12 +71,11 @@ finish_setup (CEPageWiredSecurity *self, gpointer unused, GError *error, gpointe
 {
 	CEPage *parent = CE_PAGE (self);
 	CEPageWiredSecurityPrivate *priv = CE_PAGE_WIRED_SECURITY_GET_PRIVATE (self);
-	const char *glade_file = GLADEDIR "/applet.glade";
 
 	if (error)
 		return;
 
-	priv->security = (WirelessSecurity *) ws_wpa_eap_new (glade_file, parent->connection);
+	priv->security = (WirelessSecurity *) ws_wpa_eap_new (parent->connection, TRUE);
 	if (!priv->security) {
 		g_warning ("Could not load wired 802.1x user interface.");
 		return;
@@ -85,6 +83,7 @@ finish_setup (CEPageWiredSecurity *self, gpointer unused, GError *error, gpointe
 
 	wireless_security_set_changed_notify (priv->security, stuff_changed, self);
 	priv->security_widget = wireless_security_get_widget (priv->security);
+	gtk_widget_unparent (priv->security_widget);
 
 	gtk_toggle_button_set_active (priv->enabled, priv->initial_have_8021x);
 	g_signal_connect (priv->enabled, "toggled", G_CALLBACK (enable_toggled), self);
@@ -96,20 +95,29 @@ finish_setup (CEPageWiredSecurity *self, gpointer unused, GError *error, gpointe
 }
 
 CEPage *
-ce_page_wired_security_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_wired_security_new (NMConnection *connection,
+                            GtkWindow *parent_window,
+                            const char **out_secrets_setting_name,
+                            GError **error)
 {
 	CEPageWiredSecurity *self;
-	CEPage *parent;
 	CEPageWiredSecurityPrivate *priv;
+	CEPage *parent;
+
+	self = CE_PAGE_WIRED_SECURITY (ce_page_new (CE_TYPE_PAGE_WIRED_SECURITY,
+	                                            connection,
+	                                            parent_window,
+	                                            NULL,
+	                                            NULL,
+	                                            _("802.1x Security")));
+	if (!self) {
+		g_set_error_literal (error, 0, 0, _("Could not load Wired Security security user interface."));
+		return NULL;
+	}
 
-	self = CE_PAGE_WIRED_SECURITY (g_object_new (CE_TYPE_PAGE_WIRED_SECURITY,
-	                                             CE_PAGE_CONNECTION, connection,
-	                                             CE_PAGE_PARENT_WINDOW, parent_window,
-	                                             NULL));
 	parent = CE_PAGE (self);
 	priv = CE_PAGE_WIRED_SECURITY_GET_PRIVATE (self);
 
-	parent->title = g_strdup (_("802.1x Security"));
 	parent->page = gtk_vbox_new (FALSE, 6);
 	g_object_ref_sink (G_OBJECT (parent->page));
 	gtk_container_set_border_width (GTK_CONTAINER (parent->page), 6);
@@ -120,10 +128,9 @@ ce_page_wired_security_new (NMConnection *connection, GtkWindow *parent_window,
 	priv->enabled = GTK_TOGGLE_BUTTON (gtk_check_button_new_with_label (_("Use 802.1X security for this connection")));
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, NM_SETTING_802_1X_SETTING_NAME, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
+
+	if (priv->initial_have_8021x)
+		*out_secrets_setting_name = NM_SETTING_802_1X_SETTING_NAME;
 
 	return CE_PAGE (self);
 }
@@ -141,21 +148,28 @@ validate (CEPage *page, NMConnection *connection, GError **error)
 		/* FIXME: get failed property and error out of wireless security objects */
 		valid = wireless_security_validate (priv->security, NULL);
 		if (valid) {
+			NMSetting *s_con;
+
 			/* Here's a nice hack to work around the fact that ws_802_1x_fill_connection needs wireless setting. */
 			tmp_connection = nm_connection_new ();
 			nm_connection_add_setting (tmp_connection, nm_setting_wireless_new ());
+
+			/* temp connection needs a 'connection' setting too, since most of
+			 * the EAP methods need the UUID for CA cert ignore stuff.
+			 */
+			s_con = nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+			nm_connection_add_setting (tmp_connection, nm_setting_duplicate (s_con));
+
 			ws_802_1x_fill_connection (priv->security, "wpa_eap_auth_combo", tmp_connection);
 
 			s_8021x = nm_connection_get_setting (tmp_connection, NM_TYPE_SETTING_802_1X);
 			nm_connection_add_setting (connection, NM_SETTING (g_object_ref (s_8021x)));
-			nm_gconf_copy_private_connection_values (connection, tmp_connection);
 
 			g_object_unref (tmp_connection);
 		} else
 			g_set_error (error, 0, 0, "Invalid 802.1x security");
 	} else {
 		nm_connection_remove_setting (connection, NM_TYPE_SETTING_802_1X);
-		nm_gconf_clear_private_connection_values (connection);
 		valid = TRUE;
 	}
 
