@@ -17,10 +17,11 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2005 - 2009 Red Hat, Inc.
+ * (C) Copyright 2005 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <net/ethernet.h>
@@ -44,13 +45,16 @@
 #include <nm-setting-8021x.h>
 #include <nm-setting-vpn.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-setting-cdma.h>
+#include <nm-setting-gsm.h>
+#include <nm-setting-ppp.h>
+#include <nm-setting-pppoe.h>
 #include <nm-utils.h>
-#include <nm-settings-interface.h>
 
 #include "gconf-helpers.h"
 #include "gconf-upgrade.h"
-#include "utils.h"
-#include "applet.h"
+
+#define S390_OPT_KEY_PREFIX "s390-opt-"
 
 #define DBUS_TYPE_G_ARRAY_OF_OBJECT_PATH    (dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH))
 #define DBUS_TYPE_G_ARRAY_OF_STRING         (dbus_g_type_get_collection ("GPtrArray", G_TYPE_STRING))
@@ -66,6 +70,14 @@
 #define DBUS_TYPE_G_IP6_ROUTE               (dbus_g_type_get_struct ("GValueArray", DBUS_TYPE_G_UCHAR_ARRAY, G_TYPE_UINT, DBUS_TYPE_G_UCHAR_ARRAY, G_TYPE_UINT, G_TYPE_INVALID))
 #define DBUS_TYPE_G_ARRAY_OF_IP6_ROUTE      (dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_IP6_ROUTE))
 
+#define APPLET_PREFS_PATH "/apps/nm-applet"
+
+/* The stamp is a mechanism for determining which applet version last
+ * updated GConf for various GConf update tasks in newer applet versions.
+ */
+#define APPLET_CURRENT_STAMP 3
+#define APPLET_PREFS_STAMP "/apps/nm-applet/stamp"
+
 const char *applet_8021x_cert_keys[] = {
 	"ca-cert",
 	"client-cert",
@@ -81,37 +93,40 @@ const char *vpn_ignore_keys[] = {
 	NULL
 };
 
-static PreKeyringCallback pre_keyring_cb = NULL;
-static gpointer pre_keyring_user_data = NULL;
-
-/* Sets a function to be called before each keyring access */
-void
-nm_gconf_set_pre_keyring_callback (PreKeyringCallback func, gpointer user_data)
+static GnomeKeyringAttributeList *
+_create_keyring_add_attr_list (const char *connection_uuid,
+                               const char *connection_id,
+                               const char *setting_name,
+                               const char *setting_key,
+                               char **out_display_name)
 {
-	pre_keyring_cb = func;
-	pre_keyring_user_data = user_data;
+	GnomeKeyringAttributeList *attrs = NULL;
+
+	g_return_val_if_fail (connection_uuid != NULL, NULL);
+	g_return_val_if_fail (connection_id != NULL, NULL);
+	g_return_val_if_fail (setting_name != NULL, NULL);
+	g_return_val_if_fail (setting_key != NULL, NULL);
+
+	if (out_display_name) {
+		*out_display_name = g_strdup_printf ("Network secret for %s/%s/%s",
+		                                     connection_id,
+		                                     setting_name,
+		                                     setting_key);
+	}
+
+	attrs = gnome_keyring_attribute_list_new ();
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_UUID_TAG,
+	                                            connection_uuid);
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_SN_TAG,
+	                                            setting_name);
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_SK_TAG,
+	                                            setting_key);
+	return attrs;
 }
 
-void
-pre_keyring_callback (void)
-{
-	GnomeKeyringInfo *info = NULL;
-
-	if (!pre_keyring_cb)
-		return;
-
-	/* Call the pre keyring callback if the keyring is locked or if there
-	 * was an error talking to the keyring.
-	 */
-	if (gnome_keyring_get_info_sync (NULL, &info) == GNOME_KEYRING_RESULT_OK) {
-		if (gnome_keyring_info_get_is_locked (info))
-			(*pre_keyring_cb) (pre_keyring_user_data);
-		gnome_keyring_info_free (info);
-	} else
-		(*pre_keyring_cb) (pre_keyring_user_data);
-}
-
-
 gboolean
 nm_gconf_get_int_helper (GConfClient *client,
                          const char *path,
@@ -284,6 +299,50 @@ out:
 	return success;
 }
 
+gboolean
+nm_gconf_get_stringarray_helper (GConfClient *client,
+                                 const char *path,
+                                 const char *key,
+                                 const char *setting,
+                                 GPtrArray **value)
+{
+	char *gc_key;
+	GConfValue *gc_value;
+	gboolean success = FALSE;
+
+	g_return_val_if_fail (key != NULL, FALSE);
+	g_return_val_if_fail (setting != NULL, FALSE);
+	g_return_val_if_fail (value != NULL, FALSE);
+	g_return_val_if_fail (*value == NULL, FALSE);
+
+	gc_key = g_strdup_printf ("%s/%s/%s", path, setting, key);
+	if (!(gc_value = gconf_client_get (client, gc_key, NULL)))
+		goto out;
+
+	if (gc_value->type == GCONF_VALUE_LIST
+	    && gconf_value_get_list_type (gc_value) == GCONF_VALUE_STRING)
+	{
+		GSList *iter, *list;
+		const char *string;
+
+		*value = g_ptr_array_sized_new (3);
+
+		list = gconf_value_get_list (gc_value);
+		for (iter = list; iter != NULL; iter = g_slist_next (iter)) {
+			string = gconf_value_get_string ((GConfValue *) iter->data);
+			g_ptr_array_add (*value, g_strdup (string));
+		}
+
+		success = TRUE;
+	}
+
+out:
+	if (gc_value)
+		gconf_value_free (gc_value);
+	g_free (gc_key);
+	return success;
+}
+
 typedef struct {
 	const char *setting_name;
 	const char *key_name;
@@ -530,25 +589,31 @@ nm_gconf_get_valuehash_helper (GConfClient *client,
 gboolean
 nm_gconf_get_stringhash_helper (GConfClient *client,
                                 const char *path,
+                                const char *key,
                                 const char *setting,
                                 GHashTable **value)
 {
 	char *gc_key;
 	GSList *gconf_entries;
 	GSList *iter;
-	int prefix_len;
+	int path_len;
+	const char *key_prefix = NULL;
 
 	g_return_val_if_fail (setting != NULL, FALSE);
 	g_return_val_if_fail (value != NULL, FALSE);
 
 	gc_key = g_strdup_printf ("%s/%s", path, setting);
-	prefix_len = strlen (gc_key);
+	path_len = strlen (gc_key);
 	gconf_entries = gconf_client_all_entries (client, gc_key, NULL);
 	g_free (gc_key);
 
 	if (!gconf_entries)
 		return FALSE;
 
+	if (   !strcmp (setting, NM_SETTING_WIRED_SETTING_NAME)
+	    && !strcmp (key, NM_SETTING_WIRED_S390_OPTIONS))
+		key_prefix = S390_OPT_KEY_PREFIX;
+
 	*value = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                (GDestroyNotify) g_free,
 	                                (GDestroyNotify) g_free);
@@ -557,7 +622,7 @@ nm_gconf_get_stringhash_helper (GConfClient *client,
 		GConfEntry *entry = (GConfEntry *) iter->data;
 
 		gc_key = (char *) gconf_entry_get_key (entry);
-		gc_key += prefix_len + 1; /* get rid of the full path */
+		gc_key += path_len + 1; /* get rid of the full path */
 
 		if (   !strcmp (setting, NM_SETTING_VPN_SETTING_NAME)
 		    && (!strcmp (gc_key, NM_SETTING_VPN_SERVICE_TYPE) || !strcmp (gc_key, NM_SETTING_NAME))) {
@@ -566,8 +631,20 @@ nm_gconf_get_stringhash_helper (GConfClient *client,
 			 */
 		} else {
 			GConfValue *gc_val = gconf_entry_get_value (entry);
+			gboolean ignore = FALSE;
 
-			if (gc_val) {
+			/* If we have a key prefix and the GConf item has that prefix,
+			 * strip off the prefix.  Otherwise, if the GConf items does not
+			 * have this prefix, it's not for this key and we should ignore it.
+			 */
+			if (key_prefix) {
+				if (g_str_has_prefix (gc_key, key_prefix))
+					gc_key += strlen (key_prefix);
+				else
+					ignore = TRUE;
+			}
+
+			if (gc_val && (ignore == FALSE)) {
 				const char *gc_str = gconf_value_get_string (gc_val);
 
 				if (gc_str && strlen (gc_str))
@@ -1027,6 +1104,35 @@ nm_gconf_set_stringlist_helper (GConfClient *client,
 	return TRUE;
 }
 
+gboolean
+nm_gconf_set_stringarray_helper (GConfClient *client,
+                                 const char *path,
+                                 const char *key,
+                                 const char *setting,
+                                 GPtrArray *value)
+{
+	char *gc_key;
+	GSList *list = NULL;
+	int i;
+
+	g_return_val_if_fail (key != NULL, FALSE);
+	g_return_val_if_fail (setting != NULL, FALSE);
+
+	gc_key = g_strdup_printf ("%s/%s/%s", path, setting, key);
+	if (!gc_key) {
+		g_warning ("Not enough memory to create gconf path");
+		return FALSE;
+	}
+
+	for (i = 0; i < value->len; i++)
+		list = g_slist_append (list, g_ptr_array_index (value, i));
+
+	gconf_client_set_list (client, gc_key, GCONF_VALUE_STRING, list, NULL);
+	g_slist_free (list);
+	g_free (gc_key);
+	return TRUE;
+}
+
 static gboolean
 nm_gconf_set_mac_address_helper (GConfClient *client,
                                  const char *path,
@@ -1197,47 +1303,18 @@ nm_gconf_set_valuehash_helper (GConfClient *client,
 }
 #endif
 
-static void
-write_properties_stringhash (gpointer key, gpointer value, gpointer user_data)
-{
-	WritePropertiesInfo *info = (WritePropertiesInfo *) user_data;
-	char *esc_key;
-	char *full_key;
-	const char *str_value = (const char *) value;
-
-	if (!str_value || !strlen (str_value))
-		return;
-
-	esc_key = gconf_escape_key ((char *) key, -1);
-	full_key = g_strconcat (info->path, "/", esc_key, NULL);
-	gconf_client_set_string (info->client, full_key, (char *) str_value, NULL);
-	g_free (esc_key);
-	g_free (full_key);
-}
-
-typedef struct {
-	const char *key;
-	gboolean found;
-} FindKeyInfo;
-
-static void
-find_gconf_key (gpointer key, gpointer value, gpointer user_data)
-{
-	FindKeyInfo *info = (FindKeyInfo *) user_data;
-
-	if (!info->found && !strcmp ((char *) key, info->key))
-		info->found = TRUE;
-}
-
 gboolean
 nm_gconf_set_stringhash_helper (GConfClient *client,
                                 const char *path,
+                                const char *key,
                                 const char *setting,
                                 GHashTable *value)
 {
 	char *gc_key;
 	GSList *existing, *iter;
-	WritePropertiesInfo info;
+	const char *key_prefix = NULL;
+	GHashTableIter hash_iter;
+	gpointer name, data;
 
 	g_return_val_if_fail (setting != NULL, FALSE);
 	g_return_val_if_fail (value != NULL, FALSE);
@@ -1248,28 +1325,57 @@ nm_gconf_set_stringhash_helper (GConfClient *client,
 		return FALSE;
 	}
 
+	if (   !strcmp (setting, NM_SETTING_WIRED_SETTING_NAME)
+	    && !strcmp (key, NM_SETTING_WIRED_S390_OPTIONS))
+		key_prefix = S390_OPT_KEY_PREFIX;
+
 	/* Delete GConf entries that are not in the hash table to be written */
 	existing = gconf_client_all_entries (client, gc_key, NULL);
 	for (iter = existing; iter; iter = g_slist_next (iter)) {
 		GConfEntry *entry = (GConfEntry *) iter->data;
-		char *basename = g_path_get_basename (entry->key);
-		FindKeyInfo fk_info = { basename, FALSE };
+		const char *basename = strrchr (entry->key, '/');
 
-		g_hash_table_foreach (value, find_gconf_key, &fk_info);
-		/* Be sure to never delete "special" VPN keys */
-		if (   (fk_info.found == FALSE)
-		    && strcmp ((char *) basename, NM_SETTING_VPN_SERVICE_TYPE)
-			&& strcmp ((char *) basename, NM_SETTING_VPN_USER_NAME))
-			gconf_client_unset (client, entry->key, NULL);
+		if (!basename) {
+			g_warning ("GConf key '%s' had no basename", entry->key);
+			continue;
+		}
+		basename++; /* Advance past the '/' */
+
+		/* Don't delete special VPN keys that aren't part of the VPN-plugin
+		 * specific data.
+		 */
+		if (!strcmp (setting, NM_SETTING_VPN_SETTING_NAME)) {
+			if (!strcmp (basename, NM_SETTING_VPN_SERVICE_TYPE))
+				continue;
+			if (!strcmp (basename, NM_SETTING_VPN_USER_NAME))
+				continue;
+		}
+
+		/* And if we have a key prefix, don't delete anything that does not
+		 * have the prefix.
+		 */
+		if (key_prefix && (g_str_has_prefix (basename, key_prefix) == FALSE))
+			continue;
+
+		gconf_client_unset (client, entry->key, NULL);
 		gconf_entry_unref (entry);
-		g_free (basename);
 	}
 	g_slist_free (existing);
 
 	/* Now update entries and write new ones */
-	info.client = client;
-	info.path = gc_key;
-	g_hash_table_foreach (value, write_properties_stringhash, &info);
+	g_hash_table_iter_init (&hash_iter, value);
+	while (g_hash_table_iter_next (&hash_iter, &name, &data)) {
+		char *esc_key, *full_key;
+
+		esc_key = gconf_escape_key ((char *) name, -1);
+		full_key = g_strdup_printf ("%s/%s%s",
+		                            gc_key,
+		                            key_prefix ? key_prefix : "",
+		                            esc_key);
+		gconf_client_set_string (client, full_key, (char *) data, NULL);
+		g_free (esc_key);
+		g_free (full_key);
+	}
 
 	g_free (gc_key);
 	return TRUE;
@@ -1544,13 +1650,78 @@ out:
 	return success;
 }
 
-GSList *
-nm_gconf_get_all_connections (GConfClient *client)
+
+gboolean
+nm_gconf_key_is_set (GConfClient *client,
+                     const char *path,
+                     const char *key,
+                     const char *setting)
 {
+	char *gc_key;
+	GConfValue *gc_value;
+	gboolean exists = FALSE;
+
+	g_return_val_if_fail (path != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+	g_return_val_if_fail (setting != NULL, FALSE);
+
+	gc_key = g_strdup_printf ("%s/%s/%s", path, setting, key);
+	gc_value = gconf_client_get (client, gc_key, NULL);
+	if (gc_value) {
+		exists = TRUE;
+		gconf_value_free (gc_value);
+	}
+	g_free (gc_key);
+	return exists;
+}
+
+static void
+move_to_system (GConfClient *client,
+                GSList *connections,
+                AddToSettingsFunc add_func,
+                gpointer user_data)
+{
+	GSList *iter;
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+
+	for (iter = connections; iter; iter = iter->next) {
+		connection = nm_gconf_read_connection (client, (const char *) iter->data);
+		if (!connection)
+			continue;
+
+		/* Set this connection visible only to this user */
+		s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+		g_assert (s_con);
+		nm_setting_connection_add_permission (s_con, "user", g_get_user_name (), NULL);
+
+		/* Next, any secrets for the connection need to be marked user-owned */
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_802_1X_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_CDMA_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_GSM_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_PPP_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_PPPOE_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_WIRELESS_SECURITY_SETTING_NAME);
+		nm_gconf_migrate_09_secret_flags (client, connection, NM_SETTING_VPN_SETTING_NAME);
+
+		/* Now add the connection to the system settings service */
+		if (add_func)
+			add_func (connection, user_data);
+	}
+}
+
+void
+nm_gconf_move_connections_to_system (AddToSettingsFunc add_func, gpointer user_data)
+{
+	GConfClient *client;
 	GSList *connections;
 	guint32 stamp = 0;
 	GError *error = NULL;
 
+	client = gconf_client_get_default ();
+	if (!client)
+		return;
+
 	stamp = (guint32) gconf_client_get_int (client, APPLET_PREFS_STAMP, &error);
 	if (error) {
 		g_error_free (error);
@@ -1578,14 +1749,21 @@ nm_gconf_get_all_connections (GConfClient *client)
 	connections = gconf_client_all_dirs (client, GCONF_PATH_CONNECTIONS, NULL);
 	if (!connections) {
 		nm_gconf_migrate_0_6_connections (client);
+		/* Try again */
 		connections = gconf_client_all_dirs (client, GCONF_PATH_CONNECTIONS, NULL);
 	}
 
+	if (connections) {
+		/* Move to system connections for 0.9 */
+		if (stamp < 3)
+			move_to_system (client, connections, add_func, user_data);
+		g_slist_foreach (connections, (GFunc) g_free, NULL);
+		g_slist_free (connections);
+	}
+
 	/* Update the applet GConf stamp */
 	if (stamp != APPLET_CURRENT_STAMP)
 		gconf_client_set_int (client, APPLET_PREFS_STAMP, APPLET_CURRENT_STAMP, NULL);
-
-	return connections;
 }
 
 static gboolean
@@ -1762,10 +1940,18 @@ read_one_setting_value_from_gconf (NMSetting *setting,
 	} else if (type == DBUS_TYPE_G_MAP_OF_STRING) {
 		GHashTable *sh_val = NULL;
 
-		if (nm_gconf_get_stringhash_helper (info->client, info->dir, setting_name, &sh_val)) {
+		if (nm_gconf_get_stringhash_helper (info->client, info->dir, key, setting_name, &sh_val)) {
 			g_object_set (setting, key, sh_val, NULL);
 			g_hash_table_destroy (sh_val);
 		}
+	} else if (type == DBUS_TYPE_G_ARRAY_OF_STRING) {
+		GPtrArray *pa_val = NULL;
+
+		if (nm_gconf_get_stringarray_helper (info->client, info->dir, key, setting_name, &pa_val)) {
+			g_object_set (setting, key, pa_val, NULL);
+			g_ptr_array_foreach (pa_val, (GFunc) g_free, NULL);
+			g_ptr_array_free (pa_val, TRUE);
+		}
 	} else if (type == DBUS_TYPE_G_UINT_ARRAY) {
 		GArray *a_val = NULL;
 
@@ -1887,24 +2073,12 @@ nm_gconf_add_keyring_item (const char *connection_uuid,
 	g_return_if_fail (setting_key != NULL);
 	g_return_if_fail (secret != NULL);
 
-	display_name = g_strdup_printf ("Network secret for %s/%s/%s",
-	                                connection_name,
-	                                setting_name,
-	                                setting_key);
-
-	attrs = gnome_keyring_attribute_list_new ();
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_UUID_TAG,
-	                                            connection_uuid);
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_SN_TAG,
-	                                            setting_name);
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_SK_TAG,
-	                                            setting_key);
-
-	pre_keyring_callback ();
-
+	attrs = _create_keyring_add_attr_list (connection_uuid,
+	                                       connection_name,
+	                                       setting_name,
+	                                       setting_key,
+	                                       &display_name);
+	g_assert (attrs);
 	ret = gnome_keyring_item_create_sync (NULL,
 	                                      GNOME_KEYRING_ITEM_GENERIC_SECRET,
 	                                      display_name,
@@ -1912,16 +2086,14 @@ nm_gconf_add_keyring_item (const char *connection_uuid,
 	                                      secret,
 	                                      TRUE,
 	                                      &id);
-
+	if (ret != GNOME_KEYRING_RESULT_OK) {
+		g_warning ("Failed to add keyring item (%s/%s/%s/%s): %d",
+		           connection_uuid, connection_name, setting_name, setting_key, ret);
+	}
 	gnome_keyring_attribute_list_free (attrs);
 	g_free (display_name);
 }
 
-static void
-delete_done (GnomeKeyringResult result, gpointer user_data)
-{
-}
-
 static void
 keyring_delete_item (const char *connection_uuid,
                      const char *setting_name,
@@ -1931,8 +2103,6 @@ keyring_delete_item (const char *connection_uuid,
 	GnomeKeyringResult ret;
 	GList *iter;
 
-	pre_keyring_callback ();
-
 	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
 	                                      &found_list,
 	                                      KEYRING_UUID_TAG,
@@ -1949,11 +2119,7 @@ keyring_delete_item (const char *connection_uuid,
 		for (iter = found_list; iter != NULL; iter = g_list_next (iter)) {
 			GnomeKeyringFound *found = (GnomeKeyringFound *) iter->data;
 
-			gnome_keyring_item_delete (found->keyring,
-			                           found->item_id,
-			                           delete_done,
-			                           NULL,
-			                           NULL);
+			gnome_keyring_item_delete_sync (found->keyring, found->item_id);
 		}
 		gnome_keyring_found_list_free (found_list);
 	}
@@ -2028,7 +2194,7 @@ write_secret_file (const char *path,
 
 	tmppath = g_malloc0 (strlen (path) + 10);
 	if (!tmppath) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not allocate memory for temporary file for '%s'",
 		             path);
 		return FALSE;
@@ -2040,7 +2206,7 @@ write_secret_file (const char *path,
 	errno = 0;
 	fd = mkstemp (tmppath);
 	if (fd < 0) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not create temporary file for '%s': %d",
 		             path, errno);
 		goto out;
@@ -2051,7 +2217,7 @@ write_secret_file (const char *path,
 	if (fchmod (fd, S_IRUSR | S_IWUSR)) {
 		close (fd);
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not set permissions for temporary file '%s': %d",
 		             path, errno);
 		goto out;
@@ -2062,7 +2228,7 @@ write_secret_file (const char *path,
 	if (written != len) {
 		close (fd);
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not write temporary file for '%s': %d",
 		             path, errno);
 		goto out;
@@ -2073,7 +2239,7 @@ write_secret_file (const char *path,
 	errno = 0;
 	if (rename (tmppath, path)) {
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not rename temporary file to '%s': %d",
 		             path, errno);
 		goto out;
@@ -2252,8 +2418,10 @@ write_object (GConfClient *client,
 		 * deleted, but /etc/pki/tls/cert.pem would not.
 		 */
 		standard_file = generate_cert_path (id, objtype->suffix);
-		if (g_file_test (standard_file, G_FILE_TEST_EXISTS))
+		if (g_file_test (standard_file, G_FILE_TEST_EXISTS)) {
 			ignored = unlink (standard_file);
+			if (ignored) {};  /* shut gcc up */
+		}
 		g_free (standard_file);
 
 		/* Delete the key from GConf */
@@ -2277,7 +2445,7 @@ write_object (GConfClient *client,
 
 		new_file = generate_cert_path (id, objtype->suffix);
 		if (!new_file) {
-			g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+			g_set_error (error, 0, 0,
 			             "Could not create file path for %s / %s",
 			             setting_name, objtype->setting_key);
 			return FALSE;
@@ -2292,7 +2460,7 @@ write_object (GConfClient *client,
 			nm_gconf_set_string_helper (client, dir, objtype->setting_key, setting_name, new_file);
 			return TRUE;
 		} else {
-			g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+			g_set_error (error, 0, 0,
 			             "Could not write certificate/key for %s / %s: %s",
 			             setting_name, objtype->setting_key,
 			             (write_error && write_error->message) ? write_error->message : "(unknown)");
@@ -2392,7 +2560,7 @@ write_one_certificate (GConfClient *client,
 	}
 
 	if (!handled) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Unhandled certificate/private-key item '%s'",
 		             key);
 	}
@@ -2508,9 +2676,13 @@ copy_one_setting_value_to_gconf (NMSetting *setting,
 								 (GHashTable *) g_value_get_boxed (value));
 #endif
 	} else if (type == DBUS_TYPE_G_MAP_OF_STRING) {
-		nm_gconf_set_stringhash_helper (info->client, info->dir,
+		nm_gconf_set_stringhash_helper (info->client, info->dir, key,
 		                                setting_name,
 		                                (GHashTable *) g_value_get_boxed (value));
+	} else if (type == DBUS_TYPE_G_ARRAY_OF_STRING) {
+		nm_gconf_set_stringarray_helper (info->client, info->dir, key,
+		                                setting_name,
+		                                (GPtrArray *) g_value_get_boxed (value));
 	} else if (type == DBUS_TYPE_G_UINT_ARRAY) {
 		nm_gconf_set_uint_array_helper (info->client, info->dir,
 								  key, setting_name,
@@ -2575,7 +2747,10 @@ nm_gconf_write_connection (NMConnection *connection,
 {
 	NMSettingConnection *s_con;
 	CopyOneSettingValueInfo info;
-	gboolean ignore;
+
+	/* NOTE: as of 0.9, this method should only be called during upgrade of
+	 * NM 0.6 connections.
+	 */
 
 	g_return_if_fail (NM_IS_CONNECTION (connection));
 	g_return_if_fail (client != NULL);
@@ -2608,104 +2783,5 @@ nm_gconf_write_connection (NMConnection *connection,
 		                                      write_one_secret_to_keyring,
 		                                      &info);
 	}
-
-	/* Update ignore CA cert status */
-	ignore = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), IGNORE_CA_CERT_TAG));
-	nm_gconf_set_ignore_ca_cert (info.connection_uuid, FALSE, ignore);
-
-	ignore = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), IGNORE_PHASE2_CA_CERT_TAG));
-	nm_gconf_set_ignore_ca_cert (info.connection_uuid, TRUE, ignore);
-}
-
-static char *
-get_ignore_path (const char *uuid, gboolean phase2)
-{
-	return g_strdup_printf (APPLET_PREFS_PATH "/%s/%s",
-	                        phase2 ? "ignore-phase2-ca-cert" : "ignore-ca-cert",
-	                        uuid);
-}
-
-gboolean
-nm_gconf_get_ignore_ca_cert (const char *uuid, gboolean phase2)
-{
-	GConfClient *client;
-	char *key = NULL;
-	gboolean ignore = FALSE;
-
-	g_return_val_if_fail (uuid != NULL, FALSE);
-
-	client = gconf_client_get_default ();
-
-	key = get_ignore_path (uuid, phase2);
-	ignore = gconf_client_get_bool (client, key, NULL);
-	g_free (key);
-
-	g_object_unref (client);
-	return ignore;
-}
-
-void
-nm_gconf_set_ignore_ca_cert (const char *uuid, gboolean phase2, gboolean ignore)
-{
-	GConfClient *client;
-	char *key = NULL;
-
-	g_return_if_fail (uuid != NULL);
-
-	client = gconf_client_get_default ();
-
-	key = get_ignore_path (uuid, phase2);
-	if (ignore)
-		gconf_client_set_bool (client, key, ignore, NULL);
-	else
-		gconf_client_unset (client, key, NULL);
-	g_free (key);
-
-	g_object_unref (client);
-}
-
-static char *
-get_always_ask_path (const char *uuid)
-{
-	return g_strdup_printf (APPLET_PREFS_PATH "/8021x-password-always-ask/%s", uuid);
-}
-
-gboolean
-nm_gconf_get_8021x_password_always_ask (const char *uuid)
-{
-	GConfClient *client;
-	char *key = NULL;
-	gboolean ask = FALSE;
-
-	g_return_val_if_fail (uuid != NULL, FALSE);
-
-	client = gconf_client_get_default ();
-
-	key = get_always_ask_path (uuid);
-	ask = gconf_client_get_bool (client, key, NULL);
-	g_free (key);
-
-	g_object_unref (client);
-	return ask;
-}
-
-void
-nm_gconf_set_8021x_password_always_ask (const char *uuid, gboolean ask)
-{
-	GConfClient *client;
-	char *key = NULL;
-
-	g_return_if_fail (uuid != NULL);
-
-	client = gconf_client_get_default ();
-
-	key = get_always_ask_path (uuid);
-	if (ask)
-		gconf_client_set_bool (client, key, TRUE, NULL);
-	else
-		gconf_client_unset (client, key, NULL);
-	g_free (key);
-
-	g_object_unref (client);
 }
 
