@@ -29,7 +29,6 @@
 
 #include "eap-method.h"
 #include "wireless-security.h"
-#include "gconf-helpers.h"
 
 #define I_NAME_COLUMN   0
 #define I_METHOD_COLUMN 1
@@ -147,10 +146,6 @@ fill_connection (EAPMethod *parent, NMConnection *connection)
 		g_clear_error (&error);
 	}
 
-	nm_gconf_set_ignore_ca_cert (nm_setting_connection_get_uuid (s_con),
-	                             FALSE,
-	                             eap_method_get_ignore_ca_cert (parent));
-
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter);
@@ -331,14 +326,12 @@ eap_method_ttls_new (WirelessSecurity *ws_parent,
 	                          destroy,
 	                          UIDIR "/eap-method-ttls.ui",
 	                          "eap_ttls_notebook",
-	                          "eap_ttls_anon_identity_entry");
+	                          "eap_ttls_anon_identity_entry",
+	                          FALSE);
 	if (!parent)
 		return NULL;
 
-	eap_method_nag_init (parent,
-	                     "eap_ttls_ca_cert_button",
-	                     connection,
-	                     FALSE);
+	eap_method_nag_init (parent, "eap_ttls_ca_cert_button", connection);
 
 	method = (EAPMethodTTLS *) parent;
 	method->sec_parent = ws_parent;
